@@ -1,11 +1,13 @@
 import logging
 import sys
 
-import resource as ResourceModule
+import threadpool as ThreadpoolModule
+
 
 import pomsets.error as ErrorModule
+import pomsets.library as LibraryModule
+import pomsets.resource as ResourceModule
 
-import threadpool as ThreadpoolModule
 
 class Automaton(ResourceModule.Struct):
     """
@@ -13,7 +15,7 @@ class Automaton(ResourceModule.Struct):
     """
 
     ATTRIBUTES = [
-        'executeEnvironment',
+        'executeEnvironmentMap',
         'commandManager',
         'contextManager'
     ]
@@ -157,9 +159,10 @@ class Automaton(ResourceModule.Struct):
     def enqueueRequest(self, request, shouldWait=True):
 
         # set the execute environment of the request
-        if self.executeEnvironment() is not None:
-            request.kwds['execute environment'] = self.executeEnvironment()
-        
+        if self.executeEnvironmentMap() is not None and \
+                not 'execute environment map' in request.kwds:
+            request.kwds['execute environment map'] = self.executeEnvironmentMap()
+
         threadpool = self.getThreadPoolUsingRequest(request)
 
         if threadpool.isEmpty():
@@ -274,6 +277,48 @@ class Automaton(ResourceModule.Struct):
 
         return task
 
+
+
+    def generateRequestContext(self):
+        commandBuilder = LibraryModule.CommandBuilder()
+        commandBuilderMap = {
+            'library bootstrap loader':commandBuilder,
+            'python eval':commandBuilder
+            }
+
+        executeEnvironmentMap = {
+            }
+        requestContext = {
+            'command builder map':commandBuilderMap,
+            'execute environment map':executeEnvironmentMap
+        }
+        return requestContext
+
+
+    def addValuesToRequestContextForBootstrapLoader(self, requestContext, *args, **kwds):
+
+        library = kwds['library']
+        requestContext['execute environment map']['library bootstrap loader']  =LibraryModule.LibraryLoader(library)
+        return
+
+
+    def runBootstrapLoader(self, library, isCritical=False):
+
+        definition = library.getBootstrapLoader()
+
+        import pomsets.definition as DefinitionModule
+        reference = DefinitionModule.ReferenceDefinition()
+        reference.definitionToReference(definition)
+
+        requestContext = self.generateRequestContext()
+        self.addValuesToRequestContextForBootstrapLoader(requestContext, library=library)
+
+        task = self.executePomset(pomset=reference, 
+                                  requestKwds=requestContext,
+                                  shouldWait=True)
+        return task.workRequest()
+
     
     #end class Automaton
     pass
+
