@@ -31,17 +31,23 @@ class Builder(object):
         
     def addPomsetParameter(self, pomset, parameterName, attributes):
 
+        # TODO:
+        # should verify that the pomset
+        # does not already have a parameter with the given name
+
+
         # the direction has to be specified
-        # almost everything else has a 
+        # almost everything else has a default value
         direction = attributes['direction']
 
         isOptional = attributes.get('optional', False)
-        isActive = attributes.get('active', True)
+        isActive = attributes.get('active', not isOptional)
         isCommandline = attributes.get('commandline', True)
         isFile = attributes.get('file', False)
         isList = attributes.get('list', False)
         isEnum = attributes.get('enum', False)
 
+        commandlineOptions = {}
         if isCommandline:
             prefixFlag = attributes.get('prefix flag', [])
             distributePrefixFlag = attributes.get(
@@ -66,12 +72,19 @@ class Builder(object):
             # but the name of the file is actually the input
             direction = ParameterModule.PORT_DIRECTION_INPUT
 
+        isKeyword = attributes.get('is keyword', False)
+        keywordToPass = ''
+        if isKeyword:
+            keywordToPass = attributes.get('keyword to pass')
+
         parameterAttributes = {
             ParameterModule.PORT_ATTRIBUTE_COMMANDLINE:isCommandline,
             ParameterModule.PORT_ATTRIBUTE_ISINPUTFILE:isInputFile,
             ParameterModule.PORT_ATTRIBUTE_ISSIDEEFFECT:isSideEffect,
             ParameterModule.PORT_ATTRIBUTE_ISLIST:isList,
-            ParameterModule.PORT_ATTRIBUTE_COMMANDLINE_OPTIONS:commandlineOptions
+            ParameterModule.PORT_ATTRIBUTE_COMMANDLINE_OPTIONS:commandlineOptions,
+            ParameterModule.PORT_ATTRIBUTE_KEYWORD:isKeyword,
+            ParameterModule.PORT_ATTRIBUTE_KEYWORDTOPASS:keywordToPass,
             }
 
         parameter = ParameterModule.DataParameter(
@@ -80,9 +93,14 @@ class Builder(object):
             portDirection=direction)
         parameter.name(parameterName)
 
+        defaultValue = attributes.get('default value', None)
+        if defaultValue is not None:
+            parameter.defaultValue(defaultValue)
+
         ParameterModule.setAttributes(parameter, parameterAttributes)
 
         pomset.addParameter(parameter)
+        pomset.parameterIsActive(parameterName, isActive)
 
         return parameter
 
@@ -95,8 +113,13 @@ class Builder(object):
         return
 
 
-    def createExecutableObject(self, path, staticArgs=None):
-        executableObject = TaskCommandModule.Executable()
+    def createExecutableObject(self, path, 
+                               executableClass=None, staticArgs=None):
+
+        if executableClass is None:
+            executableClass = TaskCommandModule.Executable
+        executableObject = executableClass()
+
         executableObject.stageable(False)
         executableObject.path(path)
         if staticArgs is None:
@@ -107,6 +130,8 @@ class Builder(object):
 
     def createNewAtomicPomset(self, name=None, 
                               executableObject=None,
+                              commandBuilderType=None,
+                              executeEnvironmentType=None,
                               *args, **kwds):
 
         newAtomicPomset = DefinitionModule.AtomicDefinition(*args, **kwds)
@@ -123,10 +148,15 @@ class Builder(object):
         parameterOrderings = DefinitionModule.createParameterOrderingTable()
         newAtomicPomset.parameterOrderingTable(parameterOrderings)
 
-        newAtomicPomset.commandBuilderType('shell process')
+        if commandBuilderType is None:
+            commandBuilderType = 'shell process'
+        newAtomicPomset.commandBuilderType(commandBuilderType)
 
-        newPomsetContext = ContextModule.Context()
-        newPomsetContext.pomset(newAtomicPomset)
+        if executeEnvironmentType is None:
+            executeEnvironmentType = 'shell process'
+        newAtomicPomset.executeEnvironmentType(executeEnvironmentType)
+
+        newPomsetContext = ContextModule.wrapPomsetInContext(newAtomicPomset)
         
         return newPomsetContext
 
@@ -147,8 +177,7 @@ class Builder(object):
             name = 'pomset %s' % uuid.uuid4().hex[:3]
         newPomset.name(name)
         
-        newPomsetContext = ContextModule.Context()
-        newPomsetContext.pomset(newPomset)
+        newPomsetContext = ContextModule.wrapPomsetInContext(newPomset)
         
         return newPomsetContext
 
@@ -269,6 +298,14 @@ class Builder(object):
                                  targetNode, targetParameterId)
 
 
+    def exposeNodeParameter(self, pomset, parameterId,
+                            node, nodeParameterId,
+                            shouldCreate=False):
+        return pomset.exposeNodeParameter(
+            parameterId, node, nodeParameterId, 
+            shouldCreate=shouldCreate)
+
+
     # END class Builder
     pass
 
