@@ -52,6 +52,7 @@ class ParameterBindingsHolder(ResourceModule.Struct):
     pass
 
 
+
 class Definition(ResourceModule.Struct):
     
     ATTRIBUTES = GuiOptionsHolder.ATTRIBUTES + [
@@ -62,6 +63,7 @@ class Definition(ResourceModule.Struct):
         'description',
         'name',
         'parameterOrderingTable',
+        'parameterActiveMap'
     ]
 
     SYMBOL_INPUT = 'input'
@@ -82,6 +84,7 @@ class Definition(ResourceModule.Struct):
         
         self.initializeParameters()
         self.isLibraryDefinition(False)
+        self.parameterActiveMap({})
         return
     
     
@@ -94,6 +97,7 @@ class Definition(ResourceModule.Struct):
         # than the one of the parameter?
         if id is None:
             id = parameter.id()
+
         row.setColumn('id', id)
 
         row.setColumn('parameter', parameter)
@@ -154,13 +158,13 @@ class Definition(ResourceModule.Struct):
         return filter
 
     def getParametersHavingId(self, id):
-        filter = self.getParameterIdFilter(id)
 
-        parameters = RelationalModule.Table.reduceRetrieve(
-            self.parametersTable(),
-            filter,
-            ['parameter']
-        )
+        filter = FilterModule.ObjectKeyMatchesFilter(
+            filter=FilterModule.EquivalenceFilter(id),
+            keyFunction = lambda x: x.id()
+            )
+
+        parameters = self.getParametersByFilter(filter)
         return parameters
     
     
@@ -175,8 +179,9 @@ class Definition(ResourceModule.Struct):
             raise NotImplementedError(
                 '%s has no parameter with id %s' % (self.name(), id))
         elif len(parameters) is not 1:
+
             raise NotImplementedError(
-                '%s has more than one parameter with id %s' % self.name(), id)
+                '%s has more than one parameter with id %s' % (self.name(), id))
         return parameters[0]
 
 
@@ -190,10 +195,49 @@ class Definition(ResourceModule.Struct):
             pass
         return
 
+
+    def parameterIsActive(self, id, value=None):
+        if value is not None:
+            self.parameterActiveMap()[id] = value
+        return self.parameterActiveMap().get(id, False)
     
     def isAtomic(self):
         return False
     
+
+    def getFilterForCommandlineArguments(self):
+        # TODO:
+        # should change the naming so that they're not just
+        # "command line arguments"
+        # more like "explicitly passed arguments"
+
+        parameterFilter = FilterModule.constructAndFilter()
+
+        parameterFilter.addFilter(
+            FilterModule.ObjectKeyMatchesFilter(
+                filter = FilterModule.IdentityFilter(True),
+                keyFunction = lambda x: x.getAttribute(ParameterModule.PORT_ATTRIBUTE_COMMANDLINE)
+                )
+            )
+
+        notOptionalOrActiveFilter = FilterModule.constructOrFilter()
+        notOptionalOrActiveFilter.addFilter(
+            FilterModule.ObjectKeyMatchesFilter(
+                filter = FilterModule.IdentityFilter(False),
+                keyFunction = lambda x: x.optional()
+                )
+            )
+        notOptionalOrActiveFilter.addFilter(
+            FilterModule.ObjectKeyMatchesFilter(
+                filter = FilterModule.IdentityFilter(True),
+                keyFunction = lambda x: self.parameterIsActive(x.id())
+                )
+            )
+        parameterFilter.addFilter(notOptionalOrActiveFilter)
+        return parameterFilter
+
+
+    # END class Definition
     pass
 
 
@@ -463,8 +507,7 @@ class CompositeDefinition(GraphModule.Graph, Definition,
 
         # the target parameter cannot be an input
         # nor an output file
-        if not targetParameter.portDirection() == ParameterModule.PORT_DIRECTION_INPUT or \
-                targetParameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_ISSIDEEFFECT):
+        if not targetParameter.portDirection() == ParameterModule.PORT_DIRECTION_INPUT:
             logging.debug("parameter %s is not an input" % targetParameterId)
             return False
         
@@ -475,6 +518,12 @@ class CompositeDefinition(GraphModule.Graph, Definition,
             return False
 
 
+        # we cannot connect output file to output file
+        if sourceParameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_ISSIDEEFFECT) and \
+                targetParameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_ISSIDEEFFECT):
+            logging.debug("cannot connect output file to output file")
+            return False
+
         # cannot connect if a path already exists
         filter = self.constructParameterConnectionFilter(
             sourceNode, sourceParameterId,
@@ -647,12 +696,90 @@ class CompositeDefinition(GraphModule.Graph, Definition,
         return edge
 
     
+    def exposeNodeParameter(self, 
+                            parameterId,
+                            node, nodeParameterId,
+                            shouldCreate=False):
+
+
+        nodeParameter = node.getParameter(nodeParameterId)
+        portDirection = nodeParameter.portDirection()
+
+
+        # TODO:
+        # rather than relying on a hardcoded name
+        # we should be able to figure out the actual blackboard parameter
+        # for the pomset's exposed parameter
+        blackboardParameterId = 'blackboard for %s' % parameterId
+
+
+        if shouldCreate:
+
+            # here we have to create a parameter
+            parameter = ParameterModule.DataParameter(
+                id=parameterId, optional=False, active=True,
+                portDirection=portDirection)
+            parameter.name(parameterId)
+            self.parameterIsActive(True)
+            ParameterModule.setAttributes(
+                parameter,
+                { 'commandline':False,
+                  ParameterModule.PORT_ATTRIBUTE_ISSIDEEFFECT:parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_ISSIDEEFFECT)} 
+                )
+            self.addParameter(parameter)
+
+            blackboardParameter = \
+                ParameterModule.BlackboardParameter(id=blackboardParameterId)
+            blackboardParameter.name(blackboardParameterId)
+            self.addParameter(blackboardParameter)
+
+            if portDirection == ParameterModule.PORT_DIRECTION_INPUT:
+                self._connectParameters(
+                    self, parameterId,
+                    self, blackboardParameterId)
+            else:
+                self._connectParameters(
+                    self, blackboardParameterId,
+                    self, parameterId)
+            pass
+
+
+
+        # TODO:
+        # consolidate the code in these two code paths
+        # TODO:
+        # pass the actual list of connections 
+        # for the parameter connection path
+        if portDirection == ParameterModule.PORT_DIRECTION_INPUT:
+            self._connectParameters(
+                self, blackboardParameterId,
+                node, nodeParameterId)
+            self.addParameterConnectionPath(
+                None,
+                self, parameterId,
+                node, nodeParameterId,
+                None, None)
+        else:
+            self._connectParameters(
+                node, nodeParameterId,
+                self, blackboardParameterId)
+            self.addParameterConnectionPath(
+                None,
+                node, nodeParameterId,
+                self, parameterId,
+                None, None)
+
+        return
+
+
+
     def _connectParameters(self, 
                            sourceNode, sourceParameter, 
                            targetNode, targetParameter):
         """
         This is an internal function 
         that creates a connection between two parameters
+        one of which could be an internal parameter
         """
         
         parameterConnection = ParameterModule.ParameterConnection()
@@ -763,12 +890,15 @@ class CompositeDefinition(GraphModule.Graph, Definition,
         
     
     def getMinimalNodes(self):
-        #theNotSelfFilter = self._getNotSelfParameterConnectionFilter()
-        #edges = [x for x in theNotSelfFilter.wrap(self.edges())]
-        #targetNodes = set([x.targetNode() for x in edges])
         allNodes = set(self.nodes())
 
-        filter = FilterModule.TRUE_FILTER
+        notSelfFilter = FilterModule.constructNotFilter()
+        notSelfFilter.addFilter(
+            FilterModule.IdentityFilter(self)
+            )
+        filter = RelationalModule.ColumnValueFilter(
+            'source node',
+            notSelfFilter)
         targetNodes = set(RelationalModule.Table.reduceRetrieve(
             self.parameterConnectionPathTable(),
             filter,
@@ -777,12 +907,15 @@ class CompositeDefinition(GraphModule.Graph, Definition,
         return allNodes.difference(targetNodes)
 
     def getMaximalNodes(self):
-        #theNotSelfFilter = self._getNotSelfParameterConnectionFilter()
-        #edges = [x for x in theNotSelfFilter.wrap(self.edges())]
-        #sourceNodes = set([x.sourceNode() for x in edges])
         allNodes = set(self.nodes())
 
-        filter = FilterModule.TRUE_FILTER
+        notSelfFilter = FilterModule.constructNotFilter()
+        notSelfFilter.addFilter(
+            FilterModule.IdentityFilter(self)
+            )
+        filter = RelationalModule.ColumnValueFilter(
+            'target node',
+            notSelfFilter)
         sourceNodes = set(RelationalModule.Table.reduceRetrieve(
             self.parameterConnectionPathTable(),
             filter,
@@ -894,7 +1027,8 @@ class AtomicDefinition(Definition):
     ATTRIBUTES = Definition.ATTRIBUTES + [
         'executable',
         'functionToExecute',
-        'commandBuilderType'
+        'commandBuilderType',
+        'executeEnvironmentType'
     ]
     
     def __init__(self, **kwds):
@@ -913,7 +1047,8 @@ class AtomicDefinition(Definition):
 
 
 def executeTaskInEnvironment(task, *args, **kwds):
-    environment = task.workRequest().kwds['execute environment']
+
+    environment = task.getExecuteEnvironment()
     return environment.execute(task, *args, **kwds)
 
 
@@ -923,6 +1058,7 @@ def createParameterOrderingTable():
     return table
     
 
+"""
 def createPythonEvalDefinition():
     definition = AtomicDefinition()
     definition.commandBuilderType('python eval')
@@ -940,6 +1076,7 @@ def createPythonEvalDefinition():
     definition.addParameter(parameter)
     
     return definition
+"""
 
 
 def createShellProcessDefinition(inputParameters=None,
@@ -1010,6 +1147,7 @@ def createShellProcessDefinition(inputParameters=None,
     definition.executable(executable)
     
     definition.commandBuilderType('shell process')
+    definition.executeEnvironmentType('shell process')
     
     return definition
 
@@ -1034,7 +1172,8 @@ class ReferenceDefinition(GraphModule.Node, ParameterBindingsHolder):
         'parameterSweepGroups',
         'parameterStagingMap',
         'comment',
-        'isCritical'
+        'isCritical',
+        'parameterActiveMap'
     ]
     
     def __init__(self, id=None, graph=None):
@@ -1057,6 +1196,8 @@ class ReferenceDefinition(GraphModule.Node, ParameterBindingsHolder):
         # default to True
         self.isCritical(True)
 
+        self.parameterActiveMap({})
+
         pass
 
     """
@@ -1155,7 +1296,18 @@ class ReferenceDefinition(GraphModule.Node, ParameterBindingsHolder):
         return self.definitionToReference().getParametersByFilter(theFilter)
         """
         return self.definitionToReference().getParametersByFilter(filter)
+
     
+    def parameterIsActive(self, id, value=None):
+
+        if value is not None:
+            self.parameterActiveMap()[id] = value
+
+        # if specified locally, use that value
+        # otherwise, default to value in definition
+        return self.parameterActiveMap().get(
+            id, self.definitionToReference().parameterIsActive(id))
+
     
     def getParameterToEdit(self, parameterName):
 
@@ -1188,7 +1340,21 @@ class ReferenceDefinition(GraphModule.Node, ParameterBindingsHolder):
                 FilterModule.EquivalenceFilter(parameterName)
                 )
             )
+        """
+        notGraphFilter = FilterModule.constructNotFilter()
+        notGraphFilter.addFilter(
+            RelationalModule.ColumnValueFilter(
+                'source node',
+                FilterModule.IdentityFilter(self.graph())
+                )
+            )
+        filter.addFilter(notGraphFilter)
+        """
         
+        # handle the case that this is referencing the root definition
+        if self.graph() is None:
+            return (self, parameter)
+
         sourceNode = None
         sourceParameterId = None
         for row in self.graph().parameterConnectionPathTable().retrieve(
@@ -1199,12 +1365,21 @@ class ReferenceDefinition(GraphModule.Node, ParameterBindingsHolder):
             pass
             
         if sourceNode is None:
+
+            # TODO:
+            # there are no parameter connection paths found
+            # however, see if the parameter
+            # is actually connected to a parameter
+            # in the parent definition
+            # via a blackboard parameter
+
             logging.debug('no incoming blackboard parameters found')
             return (self, parameter)
         
         parameter = sourceNode.getParameter(sourceParameterId)
         return (sourceNode, parameter)
     
+
     def referencesLibraryDefinition(self):
         definition = self.definitionToReference()
         if definition is None:
@@ -1326,40 +1501,8 @@ class ReferenceDefinition(GraphModule.Node, ParameterBindingsHolder):
         return len(groups) is 0
 
     
-    def isReadyToExecute(self, parentTask):
-        allPredecessors = set(self.predecessors())
-        
-        # need to filter for the predecessors that have completed
-        predecessorFilter = FilterModule.constructOrFilter()
-        for predecessor in allPredecessors:
-            predecessorFilter.addFilter(
-                RelationalModule.ColumnValueFilter(
-                    'definition',
-                    FilterModule.IdentityFilter(predecessor)
-                )
-            )
-            pass
-        
-        completedPredecessorFilter = FilterModule.constructAndFilter()
-        completedPredecessorFilter.addFilter(predecessorFilter)
-        completedPredecessorFilter.addFilter(
-            RelationalModule.ColumnValueFilter(
-                'status',
-                FilterModule.EquivalenceFilter('completed')
-            )
-        )
-        completedPredecessors = RelationalModule.Table.reduceRetrieve(
-            parentTask.tasksTable(),
-            completedPredecessorFilter,
-            ['definition'],
-            []
-        )
-        
-        # if every one of those predecessors
-        # can be found in tokens, and vice versa
-        return len(set(completedPredecessors).symmetric_difference(allPredecessors)) is 0
-    
-    
+
+
     def parameterStagingRequired(self, parameter, value=None):
 
         if value is not None:
@@ -1401,11 +1544,13 @@ class ReferenceDefinition(GraphModule.Node, ParameterBindingsHolder):
         nodes = set([])
         for row in self.graph().parameterConnectionPathTable().retrieve(
             filter, ['source node']):
-            targetNode = row[0]
-            if targetNode in nodes:
+            sourceNode = row[0]
+            if sourceNode in nodes:
                 continue
-            nodes.add(targetNode)
-            yield targetNode
+            if sourceNode is self.graph():
+                continue
+            nodes.add(sourceNode)
+            yield sourceNode
         raise StopIteration
 
 
@@ -1424,13 +1569,57 @@ class ReferenceDefinition(GraphModule.Node, ParameterBindingsHolder):
         nodes = set([])
         for row in self.graph().parameterConnectionPathTable().retrieve(
             filter, ['target node']):
-            sourceNode = row[0]
-            if sourceNode in nodes:
+            targetNode = row[0]
+            if targetNode in nodes:
                 continue
-            nodes.add(sourceNode)
-            yield sourceNode
+            if targetNode is self.graph():
+                continue
+            nodes.add(targetNode)
+            yield targetNode
         raise StopIteration
-        
+
+
+    def getFilterForCommandlineArguments(self):
+        # TODO:
+        # this duplicates the code in Definition.getFilterForCommandlineArguments
+        # because we have an unittest where the AtomicDefinition class
+        # needs to have this function available
+        # but by doing so, the "self.parameterIsActive"
+        # does not call the one in this class
+
+
+        # TODO:
+        # should change the naming so that they're not just
+        # "command line arguments"
+        # more like "explicitly passed arguments"
+
+        parameterFilter = FilterModule.constructAndFilter()
+
+        parameterFilter.addFilter(
+            FilterModule.ObjectKeyMatchesFilter(
+                filter = FilterModule.IdentityFilter(True),
+                keyFunction = lambda x: x.getAttribute(ParameterModule.PORT_ATTRIBUTE_COMMANDLINE)
+                )
+            )
+
+        notOptionalOrActiveFilter = FilterModule.constructOrFilter()
+        notOptionalOrActiveFilter.addFilter(
+            FilterModule.ObjectKeyMatchesFilter(
+                filter = FilterModule.IdentityFilter(False),
+                keyFunction = lambda x: x.optional()
+                )
+            )
+        notOptionalOrActiveFilter.addFilter(
+            FilterModule.ObjectKeyMatchesFilter(
+                filter = FilterModule.IdentityFilter(True),
+                keyFunction = lambda x: self.parameterIsActive(x.id())
+                )
+            )
+        parameterFilter.addFilter(notOptionalOrActiveFilter)
+        return parameterFilter
+
+
+
     
     # END class ReferenceDefinition
     pass
