@@ -19,6 +19,8 @@ import pomsets.resource as ResourceModule
 ID_LOADLIBRARYDEFINITION = 'load library definition::bb028375-bbd5-43ec-b6c3-4955c062063f'
 ID_BOOTSTRAPLOADER = 'library bootstrap loader::751fe366-1448-4db3-9db4-944075de7a5b'
 
+ID_LOADLISTVALUESFROMFILES = 'load list values from files_fc6175d6-c18b-4403-9b79-bd2b0b0012ff'
+
 
 def getBootstrapLoaderPomsetsFilter():
     # we need to filter out the bootstrap pomset loaders
@@ -53,7 +55,13 @@ class CommandBuilder(TaskModule.CommandBuilder):
 
 
 class LibraryLoader(EnvironmentModule.Environment, ResourceModule.Struct):
-    
+    """
+    NOTE: 
+    This is very similar to, and can probably subclass from,
+    cloudpool.environment.PythonEval.  We just have not 
+    spent the time to figure out how to do so yet
+    """
+
     DEFAULT_COMMANDBUILDER_TYPE = 'library bootstrap loader'
 
     ATTRIBUTES = ['library']
@@ -288,9 +296,9 @@ class Library(ResourceModule.Struct):
                 'loadLibraryDefinitions.pomset')
 
         pomset = self.generateBootstrapLoaderPomset()
-        
-        ContextModule.pickleDefinition(outputPath, pomset)
+        pomsetContext = ContextModule.wrapPomsetInContext(pomset)
 
+        ContextModule.savePomsetAs(pomsetContext, outputPath)
         return
 
     
