@@ -31,7 +31,6 @@ class Task(DefinitionModule.ParameterBindingsHolder, TaskModule.Task):
         'automaton',
     ]
     
-    
     def __init__(self):
 
         DefinitionModule.ParameterBindingsHolder.__init__(self)
@@ -62,23 +61,33 @@ class Task(DefinitionModule.ParameterBindingsHolder, TaskModule.Task):
 
 
     def pullParameterBindingsFromDefinition(self):
-        logging.debug(
-            "task %s checking if self.definition() %s has parameter bindings" %
-            (self, self.definition()))
 
         if hasattr(self.definition(), 'parameterBindings'):
-            logging.debug("pulling parameter bindings from self.definition()")
 
+            """
             for key, value in self.definition().parameterBindings().iteritems():
                 if key in self.parameterBindings():
                     continue
                 
-                logging.debug('pulled value "%s" for key %s from self.definition()' % (value, key))
+
                 self.setParameterBinding(key, value)
                 pass
-            
-        logging.debug(
-            "task %s completed checking bindings on definition" % self)
+            """
+            for parameter in self.definition().getParametersByFilter(FilterModule.TRUE_FILTER):
+                parameterId = parameter.id()
+                if parameterId in self.parameterBindings():
+                    continue
+
+                if self.definition().hasParameterBinding(parameterId):
+                    self.setParameterBinding(
+                        parameterId,
+                        self.definition().getParameterBinding(parameterId))
+                elif parameter.defaultValue() is not None:
+                    self.setParameterBinding(
+                        parameterId,
+                        parameter.defaultValue())
+                    pass
+                pass
 
         return
 
@@ -123,6 +132,13 @@ class Task(DefinitionModule.ParameterBindingsHolder, TaskModule.Task):
                 keyFunction = lambda x: x.portDirection()
             )
         )
+        theFilter.addFilter(
+            FilterModule.ObjectKeyMatchesFilter(
+                filter=FilterModule.IdentityFilter(False),
+                keyFunction = lambda x: x.optional()
+            )
+        )
+
 
         boundParameterIds = self.parameterBindings().keys()
 
@@ -149,7 +165,8 @@ class CompositeTask(Task):
         'taskGenerator',
         'tasksTable',
         'allChildTasksHaveCompleted',
-        'hasGeneratedTasks'
+        'hasGeneratedTasks',
+        'isParameterSweepTasksHolder'
     ]
     
     def __init__(self):
@@ -157,6 +174,7 @@ class CompositeTask(Task):
         
         self.allChildTasksHaveCompleted(False)
         self.initializeTasksTable()
+        self.isParameterSweepTasksHolder(False)
         return
 
     def initializeTasksTable(self):
@@ -242,6 +260,9 @@ class CompositeTask(Task):
     
     def do(self):
 
+        if self.parentTask() is not None:
+            self.parentTask().childTaskIsRunning(self)
+
         # create tasks for each of the minimal nodes in the graph
         # add those tasks to the queue
         # each of the requests being added to the queue
@@ -291,15 +312,60 @@ class CompositeTask(Task):
         return
     
     
+
+    def isReadyToExecute(self, definitionForChildTask):
+
+        if self.isParameterSweepTasksHolder():
+            parentTask = self.parentTask()
+            return parentTask.isReadyToExecute(definitionForChildTask)
+
+        allPredecessors = set(definitionForChildTask.predecessors())
+        
+        # need to filter for the predecessors that have completed
+        predecessorFilter = FilterModule.constructOrFilter()
+        for predecessor in allPredecessors:
+            predecessorFilter.addFilter(
+                RelationalModule.ColumnValueFilter(
+                    'definition',
+                    FilterModule.IdentityFilter(predecessor)
+                )
+            )
+            pass
+        
+        completedPredecessorFilter = FilterModule.constructAndFilter()
+        completedPredecessorFilter.addFilter(predecessorFilter)
+        completedPredecessorFilter.addFilter(
+            RelationalModule.ColumnValueFilter(
+                'status',
+                FilterModule.EquivalenceFilter('completed')
+            )
+        )
+        completedPredecessors = RelationalModule.Table.reduceRetrieve(
+            self.tasksTable(),
+            completedPredecessorFilter,
+            ['definition'],
+            []
+        )
+        
+        
+
+        # if every one of those predecessors
+        # can be found in tokens, and vice versa
+        readyToExecute = len(set(completedPredecessors).symmetric_difference(allPredecessors)) is 0
+
+    
+        return readyToExecute
+
+
+
     def enqueueTaskIfReady(self, definition):
 
-        #if not isinstance(definition, DefinitionModule.ReferenceDefinition):
-        #    print "skipping definition %s" % definition.name()
-        #    return
-
-        if not definition.isReadyToExecute(self):
+        if not self.isReadyToExecute(definition):
+            logging.debug("definition %s is not ready to execute" % definition.name())
             return
 
+        logging.debug("checking if status of task for definition %s is \"initialized\"" % definition.name())
+
         # retrieve all the tasks that have been initialized
         filter = FilterModule.constructAndFilter()
         filter.addFilter(self._getFilterForDefinition(definition))
@@ -342,22 +408,24 @@ class CompositeTask(Task):
 
             command.addCommand(
                 RelationalCommandModule.SetColumnValueCommand(
-                    row, 'status', 'started'
+                    row, 'status', 'queued'
                 )
             )
 
             requests.append(request)
             pass
 
+
         # execute the command
         self.automaton().executeCommand(command)
 
+        logging.debug("status of %s tasks for definition %s should now be \"queued\"" % (len(rows), definition.name()))
+
 
         # TODO:
         # should instead at this time determine 
         # the actual threadpool to place the request
         
-        
         # enqueue the requests
         map(self.automaton().enqueueRequest, requests, 
             # threadpool.wait is not re-entrant
@@ -383,13 +451,6 @@ class CompositeTask(Task):
         kwds = copy.copy(self.workRequest().kwds)
         kwds['task'] = task
 
-        # customize 'execute environment id' of kwds
-        # if it is a composite task
-        # then just use the parent task's value
-        # otherwise, should get the value from the task
-        # and look into kwds to get the actual id
-        
-        
         automaton = self.automaton()
         callback = automaton.getPostExecuteCallbackFor(task)
         exc_callback = automaton.getErrorCallbackFor(task)
@@ -501,6 +562,7 @@ class CompositeTask(Task):
         
         return
 
+
     def getTaskInformation(self, task):
         filter = self._getFilterForTask(task)
         
@@ -510,6 +572,17 @@ class CompositeTask(Task):
         row = rows[0]
         return row
     
+
+    def childTaskIsRunning(self, task):
+        row = self.getTaskInformation(task)
+        
+        command = RelationalCommandModule.SetColumnValueCommand(
+            row, 'status', 'running'
+        )
+        
+        self.automaton().executeCommand(command)
+        return
+
     def childTaskHasCompleted(self, task):
         row = self.getTaskInformation(task)
         
@@ -520,9 +593,10 @@ class CompositeTask(Task):
         self.automaton().executeCommand(command)
 
         # TODO: add this as a critical section
-        self.completedChildTask = task
-        if self.taskGenerator().canGenerateMoreTasks(self):
-            self.taskGenerator().generateReadyTasks(self)
+        #self.completedChildTask = task
+        if self.taskGenerator().canGenerateMoreTasks(self, completedChildTask=task):
+
+            self.taskGenerator().generateReadyTasks(self, completedChildTask=task)
             self.startNextTasks()
         elif self.taskGenerator().allGeneratedTasksHaveExecuted(self):
             # a maximal node just completed
@@ -532,7 +606,7 @@ class CompositeTask(Task):
             pass
             
             pass
-        del self.completedChildTask
+        #del self.completedChildTask
         
         return
 
@@ -610,6 +684,9 @@ class AtomicTask(Task):
     
     def do(self):
 
+        if self.parentTask() is not None:
+            self.parentTask().childTaskIsRunning(self)
+
         self.configureExecuteEnvironment()
         
         self.pullDataForParameters()
@@ -632,19 +709,44 @@ class AtomicTask(Task):
         that is assigned to the task
         """
         request = self.workRequest()
-        if not request.kwds.get(
-            'worker thread configures execute environment', False):
-            return
+
+        definition = self.definition()
+        if isinstance(definition, DefinitionModule.ReferenceDefinition):
+            definition = definition.definitionToReference()
+
+        executeEnvironmentType = definition.executeEnvironmentType()
+        workerThreadConfiguresEnvironment = request.kwds.get(
+            'worker thread configures execute environment', False)
+
+        if executeEnvironmentType == 'shell process' and \
+                workerThreadConfiguresEnvironment:
+
+            # the thread contains the shell
+            # to the particular host
             
-        workerThread = request.kwds.get('worker thread', None)
-        if workerThread is None:
-            raise KeyError('need worker thread to configure environment')
-        
-        request.kwds['execute environment'] = workerThread.executeEnvironment()
-        
+            workerThread = request.kwds.get('worker thread', None)
+            if workerThread is None:
+                raise KeyError('need worker thread to configure environment')
+
+            request.kwds['execute environment'] = \
+                workerThread.executeEnvironment()
+        else:
+
+            executeEnvironmentMap = request.kwds['execute environment map']
+
+            key = definition.executeEnvironmentType()
+            if key not in executeEnvironmentMap:
+                raise KeyError('no execute environment for type %s' % key)
+            request.kwds['execute environment'] = executeEnvironmentMap[key]
+
         return
 
+
     def getCommandBuilderType(self):
+        """
+        this is called by Task.getCommandBuilder()
+        """
+
         definition = self.definition()
         if isinstance(definition, DefinitionModule.ReferenceDefinition):
             definition = definition.definitionToReference()
@@ -659,8 +761,8 @@ class AtomicTask(Task):
         if not executable.stageable():
             return
         
-        request = self.workRequest()
-        shell = request.kwds['execute environment']
+        shell = self.getExecuteEnvironment()
+
         fs = shell.getFS()
 
         localPath = executable.path()
@@ -704,9 +806,8 @@ class AtomicTask(Task):
             )
         )
         
-        request = self.workRequest()
-        shell = request.kwds['execute environment']
-        # map(shell.stageFile, self.definition().getParametersByFilter(filter))
+        shell = self.getExecuteEnvironment()
+
         for parameter in self.definition().getParametersByFilter(filter):
             values = self.getParameterBinding(parameter.id())
             for value in values:
@@ -735,9 +836,8 @@ class AtomicTask(Task):
             )
         )
         
-        request = self.workRequest()
-        shell = request.kwds['execute environment']
-        # map(shell.stageFile, self.definition().getParametersByFilter(filter))
+        shell = self.getExecuteEnvironment()
+
         for parameter in self.definition().getParametersByFilter(filter):
             values = self.getParameterBinding(parameter.id())
             for value in values:
@@ -770,23 +870,27 @@ class TaskGenerator(ResourceModule.Struct):
         # the target node is self, 
         # and the target parameter is of type blackboard
         theParameterConnectionFilter = parentTask.getFilterForOwnInternalParameterConnections()
+
+
         for parameterConnection in RelationalModule.Table.reduceRetrieve(
             parentTask.definition().parameterConnectionsTable(),
             theParameterConnectionFilter,
             ['parameter connection']):
-            targetId = parameterConnection.targetParameter()
-            targetParameter = parentTask.definition().getParameter(targetId)
+
+            targetParameterId = parameterConnection.targetParameter()
+
+            targetParameter = parentTask.definition().getParameter(targetParameterId)
             if not targetParameter.portType() == ParameterModule.PORT_TYPE_BLACKBOARD:
                 continue
-            sourceId = parameterConnection.sourceParameter()
+            sourceParameterId = parameterConnection.sourceParameter()
+
+
             parentTask.setParameterBinding(
-                targetId,
-                parentTask.getParameterBinding(sourceId)
+                targetParameterId,
+                parentTask.getParameterBinding(sourceParameterId)
             )
             pass
         
-
-
         return
 
     
@@ -955,6 +1059,7 @@ class TaskGenerator(ResourceModule.Struct):
             parentTaskDefinition.parameterConnectionsTable(),
             filter,
             ['parameter connection'], [])
+
         for sourceNode, sourceParameterId, targetNode, targetParameterId, parameterConnection in parentTaskDefinition.parameterConnectionsTable().retrieve(
             filter, 
             ['source node', 'source parameter', 'target node', 'target parameter', 'parameter connection']):
@@ -993,13 +1098,14 @@ class NestTaskGenerator(TaskGenerator):
         return createTaskForDefinition(parentTask, definition)
     
     
-    def canGenerateMoreTasks(self, parentTask):
+    def canGenerateMoreTasks(self, parentTask, completedChildTask=None):
         if not hasattr(parentTask, 'hasGeneratedMinimalTasks') or \
            not parentTask.hasGeneratedMinimalTasks:
             return True
         
-        if hasattr(parentTask, 'completedChildTask'):
-            completedDefinition = parentTask.completedChildTask.definition()
+        # if hasattr(parentTask, 'completedChildTask'):
+        if completedChildTask is not None:
+            completedDefinition = completedChildTask.definition()
             if len([x for x in completedDefinition.successors()]) is not 0:
                 return True
             
@@ -1009,15 +1115,15 @@ class NestTaskGenerator(TaskGenerator):
         return False
 
     
-    def generateReadyTasks(self, parentTask):
+    def generateReadyTasks(self, parentTask, completedChildTask=None):
 
         definitionsForNextTasks = []
         if not hasattr(parentTask, 'hasGeneratedMinimalTasks') or \
            not parentTask.hasGeneratedMinimalTasks:
             definitionsForNextTasks.extend(parentTask.definition().getMinimalNodes())
             parentTask.hasGeneratedMinimalTasks=True
-        elif hasattr(parentTask, 'completedChildTask'):
-            completedDefinition = parentTask.completedChildTask.definition()
+        elif completedChildTask is not None:
+            completedDefinition = completedChildTask.definition()
             definitionsForNextTasks.extend([x for x in completedDefinition.successors()])
             pass
             
@@ -1025,6 +1131,7 @@ class NestTaskGenerator(TaskGenerator):
             definitionsForNextTasks.extend(parentTask.definitionsForNextTasks)
         
         for definitionForNextTask in definitionsForNextTasks:
+
             if not parentTask.hasInitializedChildTask(definitionForNextTask):
                 parentTask.initializeForChildDefinition(definitionForNextTask)
                 pass
@@ -1112,11 +1219,12 @@ class LoopTaskGenerator(TaskGenerator):
         self.loopIndex(0)
         return
     
-    def canGenerateMoreTasks(self, parentTask):
+    def canGenerateMoreTasks(self, parentTask, completedChildTask=None):
         
         # increment the state
         # if the previous task just completed
-        if hasattr(parentTask, 'completedChildTask') and \
+        # if hasattr(parentTask, 'completedChildTask') and \
+        if completedChildTask is not None and \
            not self.hasTransitionedState():
             # here we have to process the state transition
             stateTransitionFunction = eval(
@@ -1150,7 +1258,7 @@ class LoopTaskGenerator(TaskGenerator):
         return shouldContinue
 
     
-    def generateReadyTasks(self, parentTask):
+    def generateReadyTasks(self, parentTask, completedChildTask=None):
         
         # the initial state does not satisfy the continue condition
         if not self.canGenerateMoreTasks(parentTask):
@@ -1248,7 +1356,7 @@ class BranchTaskGenerator(TaskGenerator):
         return createTaskForDefinition(parentTask, definition)
 
     
-    def canGenerateMoreTasks(self, parentTask):
+    def canGenerateMoreTasks(self, parentTask, completedChildTask=None):
         """
         a branch is only executed once
         """
@@ -1256,9 +1364,10 @@ class BranchTaskGenerator(TaskGenerator):
             return True
         return False
 
-    def generateReadyTasks(self, parentTask):
+    def generateReadyTasks(self, parentTask, completedChildTask=None):
         
-        if not self.canGenerateMoreTasks(parentTask):
+        if not self.canGenerateMoreTasks(
+            parentTask, completedChildTask=completedChildTask):
             return
         
         conditionFunction = eval(parentTask.getParameterBinding(
@@ -1365,20 +1474,21 @@ class ParameterSweepTaskGenerator(TaskGenerator):
         self.taskTable(table)
         return    
     
-    def canGenerateMoreTasks(self, parentTask):
+    def canGenerateMoreTasks(self, parentTask, completedChildTask=None):
         if not parentTask.hasGeneratedTasks():
             return True
         return False
     
     
-    def generateReadyTasks(self, parentTask):
+    def generateReadyTasks(self, parentTask, completedChildTask=None):
 
         definitionsForNextTasks = []
         
         if not parentTask.hasGeneratedTasks():
             definitionForTask = parentTask.definition()
             
-            # here we need to create new tasks that are not Parameter Sweep Tasks
+            # here we need to create new tasks 
+            # that are not Parameter Sweep Tasks
             # but use the same definition
             # we will need to bind the correct values to the parameters
             # CODE HERE
@@ -1415,8 +1525,11 @@ class ParameterSweepTaskGenerator(TaskGenerator):
                 pass
             
             parentTask.hasGeneratedTasks(True)
+            parentTask.isParameterSweepTasksHolder(True)
             pass
+
         return
+
             
     def pullDataForParametersOfChild(self, parentTask, childTask):
         """
