@@ -12,11 +12,13 @@ import pomsets.context as ContextModule
 import pomsets.definition as DefinitionModule
 import pomsets.library as DefinitionLibraryModule
 import pomsets.parameter as ParameterModule
+import pomsets.python as PythonModule
 import pomsets.task as TaskModule
 
 ID_WORDCOUNT = 'word count_8613fe86-e7fc-4487-b4d2-0989706f8825'
 ID_WORDCOUNT_REDUCE = 'word count reducer_08979d5f-6c0d-43b7-9206-dfe69eae6c26'
 
+
 def pickleAndReloadDefinition(path, definition):
 
     # try pickling the definition
@@ -24,16 +26,16 @@ def pickleAndReloadDefinition(path, definition):
     filesToDelete = []
     try:
 
-        ContextModule.pickleDefinition(path, definition)
+        pomsetContext = ContextModule.wrapPomsetInContext(definition)
+        ContextModule.savePomsetAs(pomsetContext, path)
 
         filesToDelete.append(path)
 
         pomsetContext = ContextModule.loadPomset(path)
-        definition = pomsetContext.pomset()
-
+        definition = pomsetContext.reference()
     except Exception, e:
         logging.error("errored with msg >> %s" % e)
-        pass
+        raise
     finally:
         for fileToDelete in filesToDelete:
             if os.path.exists(fileToDelete):
@@ -44,6 +46,77 @@ def pickleAndReloadDefinition(path, definition):
     return definition
 
 
+def createLoadListValuesFromFilesDefinition():
+    builder = BuilderModule.Builder()
+
+    path = ['pomsets.python.operator',
+            'loadListValuesFromFiles']
+    executableObject = builder.createExecutableObject(
+        path,
+        executableClass=PythonModule.Function)
+
+    pythonEvalContext = builder.createNewAtomicPomset(
+        name='load list values from files',
+        executableObject = executableObject,
+        commandBuilderType = 'python eval',
+        executeEnvironmentType = 'python eval')
+    
+    definition= pythonEvalContext.pomset()
+    builder.addPomsetParameter(
+        definition, 'eval result',
+        {'direction':ParameterModule.PORT_DIRECTION_OUTPUT,
+         'commandline':False})
+    builder.addPomsetParameter(
+        definition, 'file to read',
+        {'direction':ParameterModule.PORT_DIRECTION_INPUT})
+
+    definition.id(DefinitionLibraryModule.ID_LOADLISTVALUESFROMFILES)
+    definition.isLibraryDefinition(True)
+
+    return definition
+
+def createRangeDefinition():
+
+    builder = BuilderModule.Builder()
+    path = (None, 'range')
+    executableObject = builder.createExecutableObject(
+        path,
+        executableClass=PythonModule.Function)
+
+    pythonEvalContext = builder.createNewAtomicPomset(
+        name='range',
+        executableObject = executableObject,
+        commandBuilderType = 'python eval',
+        executeEnvironmentType = 'python eval')
+    
+    definition= pythonEvalContext.pomset()
+    builder.addPomsetParameter(
+        definition, 'eval result',
+        {'direction':ParameterModule.PORT_DIRECTION_OUTPUT,
+         'commandline':False})
+    builder.addPomsetParameter(
+        definition, 'start',
+        {'direction':ParameterModule.PORT_DIRECTION_INPUT,
+         'optional':True,
+         'default value':0})
+    builder.addPomsetParameter(
+        definition, 'end',
+        {'direction':ParameterModule.PORT_DIRECTION_INPUT})
+    builder.addPomsetParameter(
+        definition, 'step',
+        {'direction':ParameterModule.PORT_DIRECTION_INPUT,
+         'optional':True})
+
+    builder.addParameterOrdering(definition, 'start', 'end')
+    builder.addParameterOrdering(definition, 'end', 'step')
+
+    definition.id(DefinitionLibraryModule.ID_LOADLISTVALUESFROMFILES)
+    definition.isLibraryDefinition(True)
+
+    return definition
+
+
+
 def createWordCountDefinition(dir=None):
     
     parameterOrdering = DefinitionModule.createParameterOrderingTable()
