@@ -2,8 +2,11 @@ import os
 import sys
 import uuid
 
+import optparse as OptparseModule
+
 sys.path.insert(0, os.getenv('POMSETS_HOME'))
 
+import pomsets.builder as BuilderModule
 import pomsets.context as ContextModule
 import pomsets.definition as DefinitionModule
 import pomsets.library as LibraryModule
@@ -15,10 +18,25 @@ import pomsets.test_utils as DefinitionTestModule
 
 
 def generateBootstrapper():
+
+    """
     defToLoadDef = DefinitionModule.AtomicDefinition()
-    defToLoadDef.commandBuilderType('python eval')
+    defToLoadDef.commandBuilderType('library bootstrap loader')
+    defToLoadDef.executeEnvironmentType('library bootstrap loader')
     defToLoadDef.id(LibraryModule.ID_LOADLIBRARYDEFINITION)
     defToLoadDef.name('load library definition')
+    """
+
+    builder = BuilderModule.Builder()
+    definitionContext = builder.createNewAtomicPomset(
+        name='load library definition', 
+        executableObject=None,
+        commandBuilderType='library bootstrap loader',
+        executeEnvironmentType='library bootstrap loader')
+    definition = definitionContext.pomset()
+    definition.id(LibraryModule.ID_LOADLIBRARYDEFINITION)
+
+    """
     # need a command builder to call the loadPomset function
     # need a python eval environment to execute the output of commandbuilder
     parameter = ParameterModule.DataParameter(
@@ -26,9 +44,16 @@ def generateBootstrapper():
         portDirection=ParameterModule.PORT_DIRECTION_INPUT)
     ParameterModule.setAttributes(parameter, {})
     defToLoadDef.addParameter(parameter)
-    defToLoadDef.isLibraryDefinition(True)
-    defToLoadDef.functionToExecute(DefinitionModule.executeTaskInEnvironment)
-    return defToLoadDef
+    """
+    builder.addPomsetParameter(
+        definition, 'pomset url', 
+        { 'direction':ParameterModule.PORT_DIRECTION_INPUT }
+        )
+
+
+    definition.isLibraryDefinition(True)
+    definition.functionToExecute(DefinitionModule.executeTaskInEnvironment)
+    return definition
 
 
 
@@ -39,30 +64,35 @@ def generateDefaultLoader(outputDir):
     defToLoadDef = generateBootstrapper()
     ContextModule.pickleDefinition(
         os.path.join(outputDir, 'loadLibraryDefinition.pomset'), defToLoadDef)
-    context = ContextModule.Context()
-    context.pomset(defToLoadDef)
+    context = ContextModule.wrapPomsetInContext(defToLoadDef)
     definitionsToLoad.append(context)
     
     wcDefinition = DefinitionTestModule.createWordCountDefinition()
-    # wcDefinitionPath = 'wordcount.pomset'
     wcDefinitionPath = wcDefinition.id() + '.pomset'
     wcDefinition.url(wcDefinitionPath)
     ContextModule.pickleDefinition(
         os.path.join(outputDir, wcDefinitionPath), wcDefinition)
-    context = ContextModule.Context()
-    context.pomset(wcDefinition)
+    context = ContextModule.wrapPomsetInContext(wcDefinition)
     definitionsToLoad.append(context)
     
     wcrDefinition = DefinitionTestModule.createWordCountReduceDefinition()
-    # wcrDefinitionPath = 'wordcount_reduce.pomset'
     wcrDefinitionPath = wcrDefinition.id() + '.pomset'
     wcrDefinition.url(wcrDefinitionPath)
     ContextModule.pickleDefinition(
         os.path.join(outputDir, wcrDefinitionPath), wcrDefinition)
-    context = ContextModule.Context()
-    context.pomset(wcrDefinition)
+    context = ContextModule.wrapPomsetInContext(wcrDefinition)
     definitionsToLoad.append(context)
-    
+
+
+    loadValuesDefinition = \
+        DefinitionTestModule.createLoadListValuesFromFilesDefinition()
+    loadValuesDefinitionPath = loadValuesDefinition.id() + '.pomset'
+    loadValuesDefinition.url(loadValuesDefinitionPath)
+    ContextModule.pickleDefinition(
+        os.path.join(outputDir, loadValuesDefinitionPath), loadValuesDefinition)
+    context = ContextModule.wrapPomsetInContext(loadValuesDefinition)
+    definitionsToLoad.append(context)
+
     library = LibraryModule.Library()
     map(library.addPomsetContext, definitionsToLoad)
 
@@ -72,6 +102,7 @@ def generateDefaultLoader(outputDir):
     
     return
 
+
 def generateLoaderWithFailure1(outputDir):
     
     definitionsToLoad = []
@@ -79,21 +110,15 @@ def generateLoaderWithFailure1(outputDir):
     defToLoadDef = generateBootstrapper()
     ContextModule.pickleDefinition(
         os.path.join(outputDir, 'loadLibraryDefinition.pomset'), defToLoadDef)
-    context = ContextModule.Context()
-    context.pomset(defToLoadDef)
+    context = ContextModule.wrapPomsetInContext(defToLoadDef)
     definitionsToLoad.append(context)
     
     wcDefinition = DefinitionTestModule.createWordCountDefinition()
-    # wcDefinition.id(ID_WORDCOUNT)
-    # wcDefinitionPath = 'wordcount.pomset'
     wcDefinitionPath = wcDefinition.id() + '.pomset'
     wcDefinition.url(wcDefinitionPath)
     # we purposely do not pickle it
-    # to ensure that the loading fi
-    #ContextModule.pickleDefinition(
-    #    os.path.join(outputDir, wcDefinitionPath), wcDefinition)
-    context = ContextModule.Context()
-    context.pomset(wcDefinition)
+    # to ensure that the loading fails
+    context = ContextModule.wrapPomsetInContext(wcDefinition)
     definitionsToLoad.append(context)
     
     library = LibraryModule.Library()
@@ -114,32 +139,23 @@ def generateLoaderWithFailure2(outputDir):
     defToLoadDef = generateBootstrapper()
     ContextModule.pickleDefinition(
         os.path.join(outputDir, 'loadLibraryDefinition.pomset'), defToLoadDef)
-    context = ContextModule.Context()
-    context.pomset(defToLoadDef)
+    context = ContextModule.wrapPomsetInContext(defToLoadDef)
     definitionsToLoad.append(context)
     
     wcDefinition = DefinitionTestModule.createWordCountDefinition()
-    # wcDefinition.id(ID_WORDCOUNT)
-    # wcDefinitionPath = 'wordcount.pomset'
     wcDefinitionPath = wcDefinition.id() + '.pomset'
     wcDefinition.url(wcDefinitionPath)
     # we purposely do not pickle it
-    # to ensure that the loading fi
-    #ContextModule.pickleDefinition(
-    #    os.path.join(outputDir, wcDefinitionPath), wcDefinition)
-    context = ContextModule.Context()
-    context.pomset(wcDefinition)
+    # to ensure that the loading fails
+    context = ContextModule.wrapPomsetInContext(wcDefinition)
     definitionsToLoad.append(context)
     
     wcrDefinition = DefinitionTestModule.createWordCountReduceDefinition()
-    # wcrDefinition.id(ID_WORDCOUNT_REDUCE)
-    # wcrDefinitionPath = 'wordcount_reduce.pomset'
     wcrDefinitionPath = wcrDefinition.id() + '.pomset'
     wcrDefinition.url(wcrDefinitionPath)
     ContextModule.pickleDefinition(
         os.path.join(outputDir, wcrDefinitionPath), wcrDefinition)
-    context = ContextModule.Context()
-    context.pomset(wcrDefinition)
+    context = ContextModule.wrapPomsetInContext(wcrDefinition)
     definitionsToLoad.append(context)
     
     library = LibraryModule.Library()
@@ -153,34 +169,28 @@ def generateLoaderWithFailure2(outputDir):
 
 
 
-def main(argv):
+def main(args):
 
-    if argv is None:
-        argv = []
+    parser = OptparseModule.OptionParser()
+    parser.add_option("--default", dest="default",
+                      help="if should generate default loader")
+    parser.add_option("--failure1", dest="failure1",
+                      help="if should generate loader with failure test case 1")
+    parser.add_option("--failure2", dest="failure2",
+                      help="if should generate loader with failure test case 2")
+    parser.add_option("-o", "--output", dest="outputDir",
+                      help="output dir")
 
-    shouldGenerateDefaultLoader = True
-    shouldGenerateLoaderWithFailure1 = False
-    shouldGenerateLoaderWithFailure2 = False
-    minArgLen = 2
+    (options, args) = parser.parse_args(args)
 
-    for arg in argv:
-        if arg.startswith('-default='):
-            minArgLen = minArgLen + 1
-            shouldGenerateDefaultLoader = arg[9:] in ['True', 'true', '1']
-        if arg.startswith('-failure1='):
-            minArgLen = minArgLen + 1
-            shouldGenerateLoaderWithFailure1 = arg[10:] in ['True', 'true', '1']
-        if arg.startswith('-failure2='):
-            minArgLen = minArgLen + 1
-            shouldGenerateLoaderWithFailure2 = arg[10:] in ['True', 'true', '1']
-        pass
+    shouldGenerateDefaultLoader = options.default is not None
+    shouldGenerateLoaderWithFailure1 = options.failure1 is not None
+    shouldGenerateLoaderWithFailure2 = options.failure2 is not None
 
-
-    if len(argv) < minArgLen:
+    outputDir = options.outputDir
+    if outputDir is None:
         raise ValueError('need to specify directory to output the definitions')
         
-    outputDir = argv[-1]
-
     if shouldGenerateDefaultLoader:
         generateDefaultLoader(outputDir)
         
