@@ -19,7 +19,6 @@ import pypatterns.filter as FilterModule
 import pomsets.command as TaskCommandModule
 import pomsets.definition as DefinitionModule
 import pomsets.error as ErrorModule
-import pomsets.library as DefinitionLibraryModule
 import pomsets.parameter as ParameterModule
 import pomsets.task as TaskModule
 
@@ -83,29 +82,31 @@ class BaseTestClass(object):
         }
         return commandBuilderMap
 
-    def createExecuteEnvironment(self):
-        return ShellModule.LocalShell()
+    def createExecuteEnvironmentMap(self):
+        return {
+            'shell process':ShellModule.LocalShell()
+            }
 
-    
-    def getRequestContext(self, task=None, executeEnvironment=None):
+
+    def getRequestContext(self, task=None, executeEnvironmentMap=None):
         
         automaton = self.automaton
         commandBuilderMap = self.createCommandBuilderMap()
 
-        if executeEnvironment is None:
-            executeEnvironment = self.createExecuteEnvironment()
+        if executeEnvironmentMap is None:
+            executeEnvironmentMap = self.createExecuteEnvironmentMap()
 
         requestContext = {
             'task':task,
             'command builder map':commandBuilderMap,
-            'execute environment':executeEnvironment
+            'execute environment map':executeEnvironmentMap
         }
         
         return requestContext
     
     
     def executeTask(self, task, 
-                    executeEnvironment=None):
+                    executeEnvironmentMap=None):
             
         automaton = self.automaton
         successCallback = automaton.getPostExecuteCallbackFor(task)
@@ -113,7 +114,7 @@ class BaseTestClass(object):
         executeTaskFunction = automaton.getExecuteTaskFunction(task)
         
         requestContext = self.getRequestContext(
-            task=task, executeEnvironment=executeEnvironment
+            task=task, executeEnvironmentMap=executeEnvironmentMap
         )
 
         request = threadpool.WorkRequest(
@@ -154,7 +155,11 @@ class BaseTestClass(object):
         self.assertPreExecute()
 
         definition = self.createDefinition()
-        task = self.createTask(definition)
+
+        reference = DefinitionModule.ReferenceDefinition()
+        reference.definitionToReference(definition)
+
+        task = self.createTask(reference)
         request = self.executeTask(task)
 
         self.request = request
@@ -171,7 +176,7 @@ class BaseTestClass(object):
         definition = self.createDefinition()
 
         definition = GeneratePomsetsModule.pickleAndReloadDefinition(
-            self.getPicklePath(),
+            '.'.join([self.getPicklePath(), 'pomset']),
             definition
         )
 
@@ -197,7 +202,9 @@ class TestCase1(BaseTestClass, unittest.TestCase):
 
     def createTask(self, definition):
         task = TaskModule.AtomicTask()
+
         task.definition(definition)
+
         task.setParameterBinding('item to echo', ['foo'])
         return task
 
@@ -219,6 +226,7 @@ class TestCase2(BaseTestClass, unittest.TestCase):
         return definition
 
     def createTask(self, definition):
+
         task = TaskModule.AtomicTask()
         task.definition(definition)
         task.setParameterBinding('item to echo', ['"echoed testExecuteAtomicFunction2"'])
@@ -256,6 +264,7 @@ class TestCase4(BaseTestClass, unittest.TestCase):
         return compositeDefinition
 
     def createTask(self, definition):
+
         compositeTask = TaskModule.CompositeTask()
         compositeTask.definition(definition)
         taskGenerator = TaskModule.NestTaskGenerator()
@@ -302,6 +311,7 @@ class TestCase8(BaseTestClass, unittest.TestCase):
         return compositeDefinition
 
     def createTask(self, definition):
+
         compositeTask = TaskModule.CompositeTask()
         compositeTask.definition(definition)
         taskGenerator = TaskModule.NestTaskGenerator()
@@ -858,17 +868,7 @@ class TestParameterSweep4(BaseTestClass, unittest.TestCase):
             compositeDefinition, 'intermediate file',
             mapperNode, 'output file'
         )
-        """
-        compositeDefinition._connectParameters(
-            compositeDefinition, 'intermediate file',
-            reducerNode, 'input files',
-        )
-        
-        compositeDefinition.connectNodes(
-            mapperNode, 'temporal output',
-            reducerNode, 'temporal input',
-        )
-        """
+
         edge = compositeDefinition.connectNodes(
             mapperNode, 'output file',
             reducerNode, 'input files',
@@ -896,14 +896,6 @@ class TestParameterSweep4(BaseTestClass, unittest.TestCase):
             'intermediate file',
             self.intermediateFiles
         )
-        """
-        print "binding %s to %s" % (self.intermediateParameterId,
-                                    self.intermediateFiles)
-        compositeTask.setParameterBinding(
-            self.intermediateParameterId,
-            self.intermediateFiles
-        )
-        """
         return compositeTask
 
 
@@ -1148,7 +1140,7 @@ class TestCase11(TestCase1):
 
     
     def executeTask(self, task, 
-                    executeEnvironment=None):
+                    executeEnvironmentMap=None):
 
         automaton = self.automaton
         successCallback = automaton.getPostExecuteCallbackFor(task)
@@ -1156,7 +1148,7 @@ class TestCase11(TestCase1):
         executeTaskFunction = automaton.getExecuteTaskFunction(task)
         
         requestContext = self.getRequestContext(
-            task=task, executeEnvironment=executeEnvironment
+            task=task, executeEnvironmentMap=executeEnvironmentMap
         )
         request = threadpool.WorkRequest(
             executeTaskFunction,
